@@ -81,7 +81,7 @@ struct _GtkCellRendererPixbufPrivate
   gchar *icon_name;
 };
 
-G_DEFINE_TYPE (GtkCellRendererPixbuf, gtk_cell_renderer_pixbuf, GTK_TYPE_CELL_RENDERER);
+G_DEFINE_TYPE (GtkCellRendererPixbuf, gtk_cell_renderer_pixbuf, GTK_TYPE_CELL_RENDERER)
 
 static void
 gtk_cell_renderer_pixbuf_init (GtkCellRendererPixbuf *cellpixbuf)
@@ -336,8 +336,7 @@ gtk_cell_renderer_pixbuf_set_property (GObject      *object,
       priv->stock_size = g_value_get_uint (value);
       break;
     case PROP_STOCK_DETAIL:
-      if (priv->stock_detail)
-        g_free (priv->stock_detail);
+      g_free (priv->stock_detail);
       priv->stock_detail = g_value_dup_string (value);
       break;
     case PROP_ICON_NAME:
@@ -439,7 +438,7 @@ gtk_cell_renderer_pixbuf_create_named_icon_pixbuf (GtkCellRendererPixbuf *cellpi
 					  priv->stock_size,
 					  &width, &height))
     {
-      g_warning ("Invalid icon size %d\n", priv->stock_size);
+      g_warning ("Invalid icon size %u\n", priv->stock_size);
       width = height = 24;
     }
 
@@ -548,25 +547,27 @@ gtk_cell_renderer_pixbuf_get_size (GtkCellRenderer *cell,
   calc_width  = (gint) cell->xpad * 2 + pixbuf_width;
   calc_height = (gint) cell->ypad * 2 + pixbuf_height;
   
-  if (x_offset) *x_offset = 0;
-  if (y_offset) *y_offset = 0;
-
   if (cell_area && pixbuf_width > 0 && pixbuf_height > 0)
     {
       if (x_offset)
 	{
 	  *x_offset = (((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) ?
-                        1.0 - cell->xalign : cell->xalign) * 
-                       (cell_area->width - calc_width - 2 * cell->xpad));
-	  *x_offset = MAX (*x_offset, 0) + cell->xpad;
+                        (1.0 - cell->xalign) : cell->xalign) * 
+                       (cell_area->width - calc_width));
+	  *x_offset = MAX (*x_offset, 0);
 	}
       if (y_offset)
 	{
 	  *y_offset = (cell->yalign *
-                       (cell_area->height - calc_height - 2 * cell->ypad));
-          *y_offset = MAX (*y_offset, 0) + cell->ypad;
+                       (cell_area->height - calc_height));
+          *y_offset = MAX (*y_offset, 0);
 	}
     }
+  else
+    {
+      if (x_offset) *x_offset = 0;
+      if (y_offset) *y_offset = 0;
+    }
 
   if (width)
     *width = calc_width;
@@ -602,8 +603,8 @@ gtk_cell_renderer_pixbuf_render (GtkCellRenderer      *cell,
 				     &pix_rect.width,
 				     &pix_rect.height);
 
-  pix_rect.x += cell_area->x;
-  pix_rect.y += cell_area->y;
+  pix_rect.x += cell_area->x + cell->xpad;
+  pix_rect.y += cell_area->y + cell->ypad;
   pix_rect.width  -= cell->xpad * 2;
   pix_rect.height -= cell->ypad * 2;
 
